#include "walking.h"
#include "darwin_kinematics.h"
#include "darwin_math.h"
#include "ram.h"
#include <math.h>

enum {PHASE0=0x00,PHASE1=0x40,PHASE2=0x80,PHASE3=0xC0};

// Walking control
uint8_t A_MOVE_AIM_ON=0x00;

// internal motion variables
float m_X_Swap_Phase_Shift;
float m_X_Swap_Amplitude;
float m_X_Swap_Amplitude_Shift;
float m_X_Move_Phase_Shift;
float m_X_Move_Amplitude;
float m_X_Move_Amplitude_Shift;
float m_Y_Swap_Phase_Shift;
float m_Y_Swap_Amplitude;
float m_Y_Swap_Amplitude_Shift;
float m_Y_Move_Phase_Shift;
float m_Y_Move_Amplitude;
float m_Y_Move_Amplitude_Shift;
float m_Z_Swap_Phase_Shift;
float m_Z_Swap_Amplitude;
float m_Z_Swap_Amplitude_Shift;
float m_Z_Move_Phase_Shift;
float m_Z_Move_Amplitude;
float m_Z_Move_Amplitude_Shift;
float m_A_Move_Phase_Shift;
float m_A_Move_Amplitude;
float m_A_Move_Amplitude_Shift;

// internal offset variables
float m_Hip_Pitch_Offset;
float m_Pelvis_Offset;
float m_Pelvis_Swing;
float m_Arm_Swing_Gain;
float m_X_Offset;
float m_Y_Offset;
float m_Z_Offset;
float m_R_Offset;
float m_P_Offset;
float m_A_Offset;

// internal time variables
float m_Time;
float m_PeriodTime;
float m_X_Swap_PeriodTime;
float m_X_Move_PeriodTime;
float m_Y_Swap_PeriodTime;
float m_Y_Move_PeriodTime;
float m_Z_Swap_PeriodTime;
float m_Z_Move_PeriodTime;
float m_A_Move_PeriodTime;
float m_SSP_Time_Start_L;
float m_SSP_Time_End_L;
float m_SSP_Time_Start_R;
float m_SSP_Time_End_R;
float m_Phase_Time1;
float m_Phase_Time2;
float m_Phase_Time3;

// control variables
uint8_t m_Ctrl_Running;
float walking_period;

// private functions
float wsin(float time, float period, float period_shift, float mag, float mag_shift)
{
  return mag*sin(((2.0*PI*time)/period)-period_shift)+mag_shift;
}

void update_param_time()
{
  float m_SSP_Ratio;
  float m_SSP_Time;

  m_PeriodTime=((float)((int16_t)(ram_data[DARWIN_WALK_PERIOD_TIME_L]+(ram_data[DARWIN_WALK_PERIOD_TIME_H]<<8))));
  m_SSP_Ratio=1.0-((float)ram_data[DARWIN_WALK_DSP_RATIO])/256.0;

  m_SSP_Time=m_PeriodTime*m_SSP_Ratio;

  m_X_Swap_PeriodTime=m_PeriodTime/2.0;
  m_X_Move_PeriodTime=m_SSP_Time;
  m_Y_Swap_PeriodTime=m_PeriodTime;
  m_Y_Move_PeriodTime=m_SSP_Time;
  m_Z_Swap_PeriodTime=m_PeriodTime/2.0;
  m_Z_Move_PeriodTime=m_SSP_Time/2.0;
  m_A_Move_PeriodTime=m_SSP_Time;

  m_SSP_Time_Start_L=(1-m_SSP_Ratio)*m_PeriodTime/4.0;
  m_SSP_Time_End_L=(1+m_SSP_Ratio)*m_PeriodTime/4.0;
  m_SSP_Time_Start_R=(3-m_SSP_Ratio)*m_PeriodTime/4.0;
  m_SSP_Time_End_R=(3+m_SSP_Ratio)*m_PeriodTime/4.0;

  m_Phase_Time1=(m_SSP_Time_End_L+m_SSP_Time_Start_L)/2.0;
  m_Phase_Time2=(m_SSP_Time_Start_R+m_SSP_Time_End_L)/2.0;
  m_Phase_Time3=(m_SSP_Time_End_R+m_SSP_Time_Start_R)/2.0;

  // m_pelvis_offset and m_pelvis_Swing in degrees
  m_Pelvis_Offset=((float)ram_data[DARWIN_WALK_PELVIS_OFFSET])*PI/1440.0;
  m_Pelvis_Swing=m_Pelvis_Offset*0.35;
  m_Arm_Swing_Gain=((float)ram_data[DARWIN_WALK_ARM_SWING_GAIN])/32.0;
}

void update_param_move()
{
  float target_x_amplitude;
  float target_y_amplitude;
  float target_a_amplitude;
  static float current_x_amplitude=0;
  static float current_y_amplitude=0;
  static float current_a_amplitude=0;

  target_x_amplitude=((float)((int8_t)ram_data[DARWIN_WALK_STEP_FW_BW]));
  // change longitudinal and transversal velocities to get to the target ones
  if(current_x_amplitude<target_x_amplitude)
  {
    current_x_amplitude+=(((float)ram_data[DARWIN_WALK_MAX_VEL])*m_PeriodTime)/1000.0;
    if(current_x_amplitude>target_x_amplitude)
      current_x_amplitude=target_x_amplitude;
  }
  else if(current_x_amplitude>target_x_amplitude)
  {
    current_x_amplitude-=(((float)ram_data[DARWIN_WALK_MAX_VEL])*m_PeriodTime)/1000.0;
    if(current_x_amplitude<target_x_amplitude)
      current_x_amplitude=target_x_amplitude;
  }
  m_X_Move_Amplitude=current_x_amplitude;
  m_X_Swap_Amplitude=current_x_amplitude*((float)ram_data[DARWIN_WALK_STEP_FW_BW_RATIO])/256.0;

  // Right/Left
  target_y_amplitude=((float)((int8_t)ram_data[DARWIN_WALK_STEP_LEFT_RIGHT]));
  if(current_y_amplitude<target_y_amplitude)
  {
    current_y_amplitude+=(((float)ram_data[DARWIN_WALK_MAX_VEL])*m_PeriodTime)/1000.0;
    if(current_y_amplitude>target_y_amplitude)
      current_y_amplitude=target_y_amplitude;
  }
  else if(current_y_amplitude>target_y_amplitude)
  {
    current_y_amplitude-=(((float)ram_data[DARWIN_WALK_MAX_VEL])*m_PeriodTime)/1000.0;
    if(current_y_amplitude<target_y_amplitude)
      current_y_amplitude=target_y_amplitude;
  }
  m_Y_Move_Amplitude=current_y_amplitude/2.0;
  if(m_Y_Move_Amplitude>0)
    m_Y_Move_Amplitude_Shift=m_Y_Move_Amplitude;
  else
    m_Y_Move_Amplitude_Shift=-m_Y_Move_Amplitude;
  m_Y_Swap_Amplitude=((float)ram_data[DARWIN_WALK_SWING_RIGHT_LEFT])+m_Y_Move_Amplitude_Shift*0.04;

  m_Z_Move_Amplitude=((float)ram_data[DARWIN_WALK_FOOT_HEIGHT])/2.0;
  m_Z_Move_Amplitude_Shift=m_Z_Move_Amplitude / 2.0;
  m_Z_Swap_Amplitude=((float)ram_data[DARWIN_WALK_SWING_TOP_DOWN]);
  m_Z_Swap_Amplitude_Shift=m_Z_Swap_Amplitude;

  // Direction
  target_a_amplitude=((float)((int8_t)ram_data[DARWIN_WALK_STEP_DIRECTION]))/8.0;
  if(current_a_amplitude<target_a_amplitude)
  {
    current_a_amplitude+=(((float)ram_data[DARWIN_WALK_MAX_ROT_VEL])*m_PeriodTime)/8000.0;
    if(current_a_amplitude>target_a_amplitude)
      current_a_amplitude=target_a_amplitude;
  }
  else if(current_a_amplitude>target_a_amplitude)
  {
    current_a_amplitude-=(((float)ram_data[DARWIN_WALK_MAX_ROT_VEL])*m_PeriodTime)/8000.0;
    if(current_a_amplitude<target_a_amplitude)
      current_a_amplitude=target_a_amplitude;
  }

  if(A_MOVE_AIM_ON==0x00)
  {
    m_A_Move_Amplitude=current_a_amplitude*PI/180.0/2.0;
    if(m_A_Move_Amplitude>0)
      m_A_Move_Amplitude_Shift=m_A_Move_Amplitude;
    else
      m_A_Move_Amplitude_Shift=-m_A_Move_Amplitude;
  }
  else
  {
    m_A_Move_Amplitude=-current_a_amplitude*PI/180.0/2.0;
    if(m_A_Move_Amplitude>0)
      m_A_Move_Amplitude_Shift=-m_A_Move_Amplitude;
    else
      m_A_Move_Amplitude_Shift=m_A_Move_Amplitude;
  }
}

void update_param_balance()
{
  m_X_Offset = ((float)((int8_t)ram_data[DARWIN_WALK_X_OFFSET]));
  m_Y_Offset = ((float)((int8_t)ram_data[DARWIN_WALK_Y_OFFSET]));
  m_Z_Offset = ((float)((int8_t)ram_data[DARWIN_WALK_Z_OFFSET]));
  m_R_Offset = ((float)((int8_t)ram_data[DARWIN_WALK_ROLL_OFFSET]))*PI/1440.0;// (r_offset/8)*(pi/180)
  m_P_Offset = ((float)((int8_t)ram_data[DARWIN_WALK_PITCH_OFFSET]))*PI/1440.0;// (p_offset/8)*(pi/180)
  m_A_Offset = ((float)((int8_t)ram_data[DARWIN_WALK_YAW_OFFSET]))*PI/1440.0;// (a_offset/8)*(pi/180)
  m_Hip_Pitch_Offset = ((float)((int16_t)(ram_data[DARWIN_WALK_HIP_PITCH_OFF_L]+(ram_data[DARWIN_WALK_HIP_PITCH_OFF_H]<<8))))/1024.0;
}

// public functions
void walking_init(uint16_t period_us)
{
  // initialize the internal motion variables
  m_X_Swap_Phase_Shift=PI;
  m_X_Swap_Amplitude_Shift=0.0;
  m_X_Move_Phase_Shift=PI/2.0;
  m_X_Move_Amplitude=0.0;
  m_X_Move_Amplitude_Shift=0.0;
  m_Y_Swap_Phase_Shift=0.0;
  m_Y_Swap_Amplitude_Shift=0.0;
  m_Y_Move_Phase_Shift=PI/2.0;
  m_Z_Swap_Phase_Shift=PI*3.0/2.0;
  m_Z_Move_Phase_Shift=PI/2.0;
  m_A_Move_Phase_Shift=PI/2.0;

  // initialize internal control variables
  m_Time=0;
  walking_period=((float)period_us)/1000.0;
  m_Ctrl_Running=0x00;

  update_param_time();
  update_param_move();

  walking_process();
}

void walking_set_period(uint16_t period_us)
{
  walking_period=((float)period_us)/1000.0;
}

uint16_t walking_get_period(void)
{
  return (uint16_t)(walking_period*1000);
}

void walking_start(void)
{
  ram_data[DARWIN_WALK_CNTRL]|=WALK_STATUS;
  m_Ctrl_Running=0x01;
}

void walking_stop(void)
{
  m_Ctrl_Running=0x00;
}

uint8_t is_walking(void)
{
  if(ram_data[DARWIN_WALK_CNTRL]&WALK_STATUS)
    return 0x01;
  else 
    return 0x00;
}

// motion manager interface functions
void walking_process(void)
{
  float x_swap,y_swap,z_swap,a_swap,b_swap,c_swap;
  float x_move_r,y_move_r,z_move_r,a_move_r,b_move_r,c_move_r;
  float x_move_l,y_move_l,z_move_l,a_move_l,b_move_l,c_move_l;
  float pelvis_offset_r,pelvis_offset_l;
//  float m_Body_Swing_Y,m_Body_Swing_Z;
  float angle[14],ep[12];

  if(m_Time==0)
  {
    update_param_time();
    ram_data[DARWIN_WALK_CNTRL]&=(~WALK_PHASE);
    ram_data[DARWIN_WALK_CNTRL]|=PHASE0;
    if(m_Ctrl_Running==0x00)
    {
      if(m_X_Move_Amplitude==0 && m_Y_Move_Amplitude==0 && m_A_Move_Amplitude==0)
        ram_data[DARWIN_WALK_CNTRL]&=(~WALK_STATUS);
      else
      {
        ram_data[DARWIN_WALK_STEP_FW_BW]=0.0;
        ram_data[DARWIN_WALK_STEP_LEFT_RIGHT]=0.0;
        ram_data[DARWIN_WALK_STEP_DIRECTION]=0.0;
      }
    }
  }
  else if(m_Time>=(m_Phase_Time1-walking_period/2.0) && m_Time<(m_Phase_Time1+walking_period/2.0))
  {
    update_param_move();
    ram_data[DARWIN_WALK_CNTRL]&=(~WALK_PHASE);
    ram_data[DARWIN_WALK_CNTRL]|=PHASE1;
  }
  else if(m_Time>=(m_Phase_Time2-walking_period/2.0) && m_Time<(m_Phase_Time2+walking_period/2.0))
  {
    update_param_time();
    m_Time=m_Phase_Time2;
    ram_data[DARWIN_WALK_CNTRL]&=(~WALK_PHASE);
    ram_data[DARWIN_WALK_CNTRL]|=PHASE2;
    if(m_Ctrl_Running==0x00)
    {
      if(m_X_Move_Amplitude==0 && m_Y_Move_Amplitude==0 && m_A_Move_Amplitude==0)
        ram_data[DARWIN_WALK_CNTRL]&=~WALK_STATUS;
      else
      {
        ram_data[DARWIN_WALK_STEP_FW_BW]=0.0;
        ram_data[DARWIN_WALK_STEP_LEFT_RIGHT]=0.0;
        ram_data[DARWIN_WALK_STEP_DIRECTION]=0.0;
      }
    }
  }
  else if(m_Time>=(m_Phase_Time3-walking_period/2.0) && m_Time<(m_Phase_Time3+walking_period/2.0))
  {
    update_param_move();
    ram_data[DARWIN_WALK_CNTRL]&=(~WALK_PHASE);
    ram_data[DARWIN_WALK_CNTRL]|=PHASE3;
  }
  update_param_balance();

  // Compute endpoints
  x_swap=wsin(m_Time,m_X_Swap_PeriodTime,m_X_Swap_Phase_Shift,m_X_Swap_Amplitude,m_X_Swap_Amplitude_Shift);
  y_swap=wsin(m_Time,m_Y_Swap_PeriodTime,m_Y_Swap_Phase_Shift,m_Y_Swap_Amplitude,m_Y_Swap_Amplitude_Shift);
  z_swap=wsin(m_Time,m_Z_Swap_PeriodTime,m_Z_Swap_Phase_Shift,m_Z_Swap_Amplitude,m_Z_Swap_Amplitude_Shift);
  a_swap=0;
  b_swap=0;
  c_swap=0;
  if(m_Time<=m_SSP_Time_Start_L)
  {
    x_move_l=wsin(m_SSP_Time_Start_L,m_X_Move_PeriodTime,m_X_Move_Phase_Shift+2.0*PI/m_X_Move_PeriodTime*m_SSP_Time_Start_L,m_X_Move_Amplitude,m_X_Move_Amplitude_Shift);
    y_move_l=wsin(m_SSP_Time_Start_L,m_Y_Move_PeriodTime,m_Y_Move_Phase_Shift+2.0*PI/m_Y_Move_PeriodTime*m_SSP_Time_Start_L,m_Y_Move_Amplitude,m_Y_Move_Amplitude_Shift);
    z_move_l=wsin(m_SSP_Time_Start_L,m_Z_Move_PeriodTime,m_Z_Move_Phase_Shift+2.0*PI/m_Z_Move_PeriodTime*m_SSP_Time_Start_L,m_Z_Move_Amplitude,m_Z_Move_Amplitude_Shift);
    c_move_l=wsin(m_SSP_Time_Start_L,m_A_Move_PeriodTime,m_A_Move_Phase_Shift+2.0*PI/m_A_Move_PeriodTime*m_SSP_Time_Start_L,m_A_Move_Amplitude,m_A_Move_Amplitude_Shift);
    x_move_r=wsin(m_SSP_Time_Start_L,m_X_Move_PeriodTime,m_X_Move_Phase_Shift+2.0*PI/m_X_Move_PeriodTime*m_SSP_Time_Start_L,-m_X_Move_Amplitude,-m_X_Move_Amplitude_Shift);
    y_move_r=wsin(m_SSP_Time_Start_L,m_Y_Move_PeriodTime,m_Y_Move_Phase_Shift+2.0*PI/m_Y_Move_PeriodTime*m_SSP_Time_Start_L,-m_Y_Move_Amplitude,-m_Y_Move_Amplitude_Shift);
    z_move_r=wsin(m_SSP_Time_Start_R,m_Z_Move_PeriodTime,m_Z_Move_Phase_Shift+2.0*PI/m_Z_Move_PeriodTime*m_SSP_Time_Start_R,m_Z_Move_Amplitude,m_Z_Move_Amplitude_Shift);
    c_move_r=wsin(m_SSP_Time_Start_L,m_A_Move_PeriodTime,m_A_Move_Phase_Shift+2.0*PI/m_A_Move_PeriodTime*m_SSP_Time_Start_L,-m_A_Move_Amplitude,-m_A_Move_Amplitude_Shift);
    pelvis_offset_l=0;
    pelvis_offset_r=0;
  }
  else if(m_Time<=m_SSP_Time_End_L)
  {
    x_move_l=wsin(m_Time,m_X_Move_PeriodTime,m_X_Move_Phase_Shift+2.0*PI/m_X_Move_PeriodTime*m_SSP_Time_Start_L,m_X_Move_Amplitude,m_X_Move_Amplitude_Shift);
    y_move_l=wsin(m_Time,m_Y_Move_PeriodTime,m_Y_Move_Phase_Shift+2.0*PI/m_Y_Move_PeriodTime*m_SSP_Time_Start_L,m_Y_Move_Amplitude,m_Y_Move_Amplitude_Shift);
    z_move_l=wsin(m_Time,m_Z_Move_PeriodTime,m_Z_Move_Phase_Shift+2.0*PI/m_Z_Move_PeriodTime*m_SSP_Time_Start_L,m_Z_Move_Amplitude,m_Z_Move_Amplitude_Shift);
    c_move_l=wsin(m_Time,m_A_Move_PeriodTime,m_A_Move_Phase_Shift+2.0*PI/m_A_Move_PeriodTime*m_SSP_Time_Start_L,m_A_Move_Amplitude,m_A_Move_Amplitude_Shift);
    x_move_r=wsin(m_Time,m_X_Move_PeriodTime,m_X_Move_Phase_Shift+2.0*PI/m_X_Move_PeriodTime*m_SSP_Time_Start_L,-m_X_Move_Amplitude,-m_X_Move_Amplitude_Shift);
    y_move_r=wsin(m_Time,m_Y_Move_PeriodTime,m_Y_Move_Phase_Shift+2.0*PI/m_Y_Move_PeriodTime*m_SSP_Time_Start_L,-m_Y_Move_Amplitude,-m_Y_Move_Amplitude_Shift);
    z_move_r=wsin(m_SSP_Time_Start_R,m_Z_Move_PeriodTime,m_Z_Move_Phase_Shift+2.0*PI/m_Z_Move_PeriodTime*m_SSP_Time_Start_R,m_Z_Move_Amplitude,m_Z_Move_Amplitude_Shift);
    c_move_r=wsin(m_Time,m_A_Move_PeriodTime,m_A_Move_Phase_Shift+2.0*PI/m_A_Move_PeriodTime*m_SSP_Time_Start_L,-m_A_Move_Amplitude,-m_A_Move_Amplitude_Shift);
    pelvis_offset_l=wsin(m_Time,m_Z_Move_PeriodTime,m_Z_Move_Phase_Shift+2.0*PI/m_Z_Move_PeriodTime*m_SSP_Time_Start_L,m_Pelvis_Swing/2.0,m_Pelvis_Swing/2.0);
    pelvis_offset_r=wsin(m_Time,m_Z_Move_PeriodTime,m_Z_Move_Phase_Shift+2.0*PI/m_Z_Move_PeriodTime*m_SSP_Time_Start_L,-m_Pelvis_Offset/2.0,-m_Pelvis_Offset/2.0);
  }
  else if(m_Time<=m_SSP_Time_Start_R)
  {
    x_move_l=wsin(m_SSP_Time_End_L,m_X_Move_PeriodTime,m_X_Move_Phase_Shift+2.0*PI/m_X_Move_PeriodTime*m_SSP_Time_Start_L,m_X_Move_Amplitude,m_X_Move_Amplitude_Shift);
    y_move_l=wsin(m_SSP_Time_End_L,m_Y_Move_PeriodTime,m_Y_Move_Phase_Shift+2.0*PI/m_Y_Move_PeriodTime*m_SSP_Time_Start_L,m_Y_Move_Amplitude,m_Y_Move_Amplitude_Shift);
    z_move_l=wsin(m_SSP_Time_End_L,m_Z_Move_PeriodTime,m_Z_Move_Phase_Shift+2.0*PI/m_Z_Move_PeriodTime*m_SSP_Time_Start_L,m_Z_Move_Amplitude,m_Z_Move_Amplitude_Shift);
    c_move_l=wsin(m_SSP_Time_End_L,m_A_Move_PeriodTime,m_A_Move_Phase_Shift+2.0*PI/m_A_Move_PeriodTime*m_SSP_Time_Start_L,m_A_Move_Amplitude,m_A_Move_Amplitude_Shift);
    x_move_r=wsin(m_SSP_Time_End_L,m_X_Move_PeriodTime,m_X_Move_Phase_Shift+2.0*PI/m_X_Move_PeriodTime*m_SSP_Time_Start_L,-m_X_Move_Amplitude,-m_X_Move_Amplitude_Shift);
    y_move_r=wsin(m_SSP_Time_End_L,m_Y_Move_PeriodTime,m_Y_Move_Phase_Shift+2.0*PI/m_Y_Move_PeriodTime*m_SSP_Time_Start_L,-m_Y_Move_Amplitude,-m_Y_Move_Amplitude_Shift);
    z_move_r=wsin(m_SSP_Time_Start_R,m_Z_Move_PeriodTime,m_Z_Move_Phase_Shift+2.0*PI/m_Z_Move_PeriodTime*m_SSP_Time_Start_R,m_Z_Move_Amplitude,m_Z_Move_Amplitude_Shift);
    c_move_r=wsin(m_SSP_Time_End_L,m_A_Move_PeriodTime,m_A_Move_Phase_Shift+2.0*PI/m_A_Move_PeriodTime*m_SSP_Time_Start_L,-m_A_Move_Amplitude,-m_A_Move_Amplitude_Shift);
    pelvis_offset_l=0.0;
    pelvis_offset_r=0.0;
  }
  else if(m_Time<=m_SSP_Time_End_R)
  {
    x_move_l=wsin(m_Time,m_X_Move_PeriodTime,m_X_Move_Phase_Shift+2.0*PI/m_X_Move_PeriodTime*m_SSP_Time_Start_R+PI,m_X_Move_Amplitude,m_X_Move_Amplitude_Shift);
    y_move_l=wsin(m_Time,m_Y_Move_PeriodTime,m_Y_Move_Phase_Shift+2.0*PI/m_Y_Move_PeriodTime*m_SSP_Time_Start_R+PI,m_Y_Move_Amplitude,m_Y_Move_Amplitude_Shift);
    z_move_l=wsin(m_SSP_Time_End_L,m_Z_Move_PeriodTime,m_Z_Move_Phase_Shift+2.0*PI/m_Z_Move_PeriodTime*m_SSP_Time_Start_L,m_Z_Move_Amplitude,m_Z_Move_Amplitude_Shift);
    c_move_l=wsin(m_Time,m_A_Move_PeriodTime,m_A_Move_Phase_Shift+2.0*PI/m_A_Move_PeriodTime*m_SSP_Time_Start_R+PI,m_A_Move_Amplitude,m_A_Move_Amplitude_Shift);
    x_move_r=wsin(m_Time,m_X_Move_PeriodTime,m_X_Move_Phase_Shift+2.0*PI/m_X_Move_PeriodTime*m_SSP_Time_Start_R+PI,-m_X_Move_Amplitude,-m_X_Move_Amplitude_Shift);
    y_move_r=wsin(m_Time,m_Y_Move_PeriodTime,m_Y_Move_Phase_Shift+2.0*PI/m_Y_Move_PeriodTime*m_SSP_Time_Start_R+PI,-m_Y_Move_Amplitude,-m_Y_Move_Amplitude_Shift);
    z_move_r=wsin(m_Time,m_Z_Move_PeriodTime,m_Z_Move_Phase_Shift+2.0*PI/m_Z_Move_PeriodTime*m_SSP_Time_Start_R,m_Z_Move_Amplitude,m_Z_Move_Amplitude_Shift);
    c_move_r=wsin(m_Time,m_A_Move_PeriodTime,m_A_Move_Phase_Shift+2.0*PI/m_A_Move_PeriodTime*m_SSP_Time_Start_R+PI,-m_A_Move_Amplitude,-m_A_Move_Amplitude_Shift);
    pelvis_offset_l=wsin(m_Time,m_Z_Move_PeriodTime,m_Z_Move_Phase_Shift+2.0*PI/m_Z_Move_PeriodTime*m_SSP_Time_Start_R,m_Pelvis_Offset/2,m_Pelvis_Offset/2);
    pelvis_offset_r=wsin(m_Time,m_Z_Move_PeriodTime,m_Z_Move_Phase_Shift+2.0*PI/m_Z_Move_PeriodTime*m_SSP_Time_Start_R,-m_Pelvis_Swing/2,-m_Pelvis_Swing/2);
  }
  else
  {
    x_move_l=wsin(m_SSP_Time_End_R,m_X_Move_PeriodTime,m_X_Move_Phase_Shift+2.0*PI/m_X_Move_PeriodTime*m_SSP_Time_Start_R+PI,m_X_Move_Amplitude,m_X_Move_Amplitude_Shift);
    y_move_l=wsin(m_SSP_Time_End_R,m_Y_Move_PeriodTime,m_Y_Move_Phase_Shift+2.0*PI/m_Y_Move_PeriodTime*m_SSP_Time_Start_R+PI,m_Y_Move_Amplitude,m_Y_Move_Amplitude_Shift);
    z_move_l=wsin(m_SSP_Time_End_L,m_Z_Move_PeriodTime,m_Z_Move_Phase_Shift+2.0*PI/m_Z_Move_PeriodTime*m_SSP_Time_Start_L,m_Z_Move_Amplitude,m_Z_Move_Amplitude_Shift);
    c_move_l=wsin(m_SSP_Time_End_R,m_A_Move_PeriodTime,m_A_Move_Phase_Shift+2.0*PI/m_A_Move_PeriodTime*m_SSP_Time_Start_R+PI,m_A_Move_Amplitude,m_A_Move_Amplitude_Shift);
    x_move_r=wsin(m_SSP_Time_End_R,m_X_Move_PeriodTime,m_X_Move_Phase_Shift+2.0*PI/m_X_Move_PeriodTime*m_SSP_Time_Start_R+PI,-m_X_Move_Amplitude,-m_X_Move_Amplitude_Shift);
    y_move_r=wsin(m_SSP_Time_End_R,m_Y_Move_PeriodTime,m_Y_Move_Phase_Shift+2.0*PI/m_Y_Move_PeriodTime*m_SSP_Time_Start_R+PI,-m_Y_Move_Amplitude,-m_Y_Move_Amplitude_Shift);
    z_move_r=wsin(m_SSP_Time_End_R,m_Z_Move_PeriodTime,m_Z_Move_Phase_Shift+2.0*PI/m_Z_Move_PeriodTime*m_SSP_Time_Start_R,m_Z_Move_Amplitude,m_Z_Move_Amplitude_Shift);
    c_move_r=wsin(m_SSP_Time_End_R,m_A_Move_PeriodTime,m_A_Move_Phase_Shift+2.0*PI/m_A_Move_PeriodTime*m_SSP_Time_Start_R+PI,-m_A_Move_Amplitude,-m_A_Move_Amplitude_Shift);
    pelvis_offset_l=0.0;
    pelvis_offset_r=0.0;
  }
  a_move_l=0.0;
  b_move_l=0.0;
  a_move_r=0.0;
  b_move_r=0.0;

  ep[0]=x_swap+x_move_r+m_X_Offset;
  ep[1]=y_swap+y_move_r-m_Y_Offset / 2.0;
  ep[2]=z_swap+z_move_r+m_Z_Offset;
  ep[3]=a_swap+a_move_r-m_R_Offset / 2.0;
  ep[4]=b_swap+b_move_r+m_P_Offset;
  ep[5]=c_swap+c_move_r-m_A_Offset / 2.0;
  ep[6]=x_swap+x_move_l+m_X_Offset;
  ep[7]=y_swap+y_move_l+m_Y_Offset / 2.0;
  ep[8]=z_swap+z_move_l+m_Z_Offset;
  ep[9]=a_swap+a_move_l+m_R_Offset / 2.0;
  ep[10]=b_swap+b_move_l+m_P_Offset;
  ep[11]=c_swap+c_move_l+m_A_Offset / 2.0;

  // Compute body swing
/*  if(m_Time<=m_SSP_Time_End_L)
  {
    m_Body_Swing_Y=-ep[7];
    m_Body_Swing_Z=ep[8];
  }
  else
  {
    m_Body_Swing_Y=-ep[1];
    m_Body_Swing_Z=ep[2];
  }
  m_Body_Swing_Z-=DARWIN_LEG_LENGTH;*/

  // Compute arm swing
  if(m_X_Move_Amplitude==0)
  {
    angle[12]=0; // Right
    angle[13]=0; // Left
  }
  else
  {
    angle[12]=wsin(m_Time,m_PeriodTime,0.0,-m_X_Move_Amplitude*m_Arm_Swing_Gain,0.0);
    angle[13]=wsin(m_Time,m_PeriodTime,0.0,m_X_Move_Amplitude*m_Arm_Swing_Gain,0.0);
  }

  if(ram_data[DARWIN_WALK_CNTRL]&WALK_STATUS)
  {
    m_Time += walking_period;
    if(m_Time >= m_PeriodTime)
      m_Time = 0;
  }

  // Compute angles with the inverse kinematics
  if((darwin_leg_ik(&angle[0],ep[0],ep[1],ep[2],ep[3],ep[4],ep[5])!=1) || 
     (darwin_leg_ik(&angle[6],ep[6],ep[7],ep[8],ep[9],ep[10],ep[11])!=1))
    return;// Do not use angles

  // Compute motor value
  if(manager_get_module(R_HIP_YAW)==MM_WALKING)
    manager_current_angles[R_HIP_YAW]=((-180.0*angle[0])/PI)*65536.0;
  if(manager_get_module(R_HIP_ROLL)==MM_WALKING)
    manager_current_angles[R_HIP_ROLL]=((-180.0*(angle[1]+pelvis_offset_r))/PI)*65536.0;
  if(manager_get_module(R_HIP_PITCH)==MM_WALKING)
    manager_current_angles[R_HIP_PITCH]=((180.0*angle[2])/PI-m_Hip_Pitch_Offset)*65536.0;
  if(manager_get_module(R_KNEE)==MM_WALKING)
    manager_current_angles[R_KNEE]=((180.0*angle[3])/PI)*65536.0;
  if(manager_get_module(R_ANKLE_PITCH)==MM_WALKING)
    manager_current_angles[R_ANKLE_PITCH]=((-180.0*angle[4])/PI)*65536.0;
  if(manager_get_module(R_ANKLE_ROLL)==MM_WALKING)
    manager_current_angles[R_ANKLE_ROLL]=((180.0*angle[5])/PI)*65536.0;
  if(manager_get_module(L_HIP_YAW)==MM_WALKING)
    manager_current_angles[L_HIP_YAW]=((-180.0*angle[6])/PI)*65536.0;
  if(manager_get_module(L_HIP_ROLL)==MM_WALKING)
    manager_current_angles[L_HIP_ROLL]=((-180.0*(angle[7]+pelvis_offset_l))/PI)*65536.0;
  if(manager_get_module(L_HIP_PITCH)==MM_WALKING)
    manager_current_angles[L_HIP_PITCH]=((-180.0*angle[8])/PI+m_Hip_Pitch_Offset)*65536.0;
  if(manager_get_module(L_KNEE)==MM_WALKING)
    manager_current_angles[L_KNEE]=((-180.0*angle[9])/PI)*65536.0;
  if(manager_get_module(L_ANKLE_PITCH)==MM_WALKING)
    manager_current_angles[L_ANKLE_PITCH]=((180.0*angle[10])/PI)*65536.0;
  if(manager_get_module(L_ANKLE_ROLL)==MM_WALKING)
    manager_current_angles[L_ANKLE_ROLL]=((180.0*angle[11])/PI)*65536.0;
  if(manager_get_module(R_SHOULDER_PITCH)==MM_WALKING)
    manager_current_angles[R_SHOULDER_PITCH]=(angle[12]-48.345)*65536.0;
  if(manager_get_module(L_SHOULDER_PITCH)==MM_WALKING)
    manager_current_angles[L_SHOULDER_PITCH]=(-angle[13]+41.313)*65536.0;
}

// operation functions
uint8_t walking_in_range(unsigned short int address, unsigned short int length)
{
  if(ram_in_window(WALK_BASE_ADDRESS,WALK_MEM_LENGTH,address,length) ||
     ram_in_window(WALK_EEPROM_ADDRESS,WALK_EEPROM_LENGTH,address,length))
    return 0x01;
  else
    return 0x00;
}

void walking_process_read_cmd(unsigned short int address,unsigned short int length,unsigned char *data)
{

}

void walking_process_write_cmd(unsigned short int address,unsigned short int length,unsigned char *data)
{
  uint16_t i;

  // walk control
  if(ram_in_range(DARWIN_WALK_CNTRL,address,length))
  {
    if(data[DARWIN_WALK_CNTRL-address]&WALK_START)
      walking_start();
    if(data[DARWIN_WALK_CNTRL-address]&WALK_STOP)
      walking_stop();
  }
  if(ram_in_range(DARWIN_WALK_STEP_FW_BW,address,length))
    ram_data[DARWIN_WALK_STEP_FW_BW]=data[DARWIN_WALK_STEP_FW_BW-address];
  if(ram_in_range(DARWIN_WALK_STEP_LEFT_RIGHT,address,length))
    ram_data[DARWIN_WALK_STEP_LEFT_RIGHT]=data[DARWIN_WALK_STEP_LEFT_RIGHT-address];
  if(ram_in_range(DARWIN_WALK_STEP_DIRECTION,address,length))
    ram_data[DARWIN_WALK_STEP_DIRECTION]=data[DARWIN_WALK_STEP_DIRECTION-address];
  // walk parameters
  for(i=WALK_EEPROM_ADDRESS;i<=WALK_EEPROM_ADDRESS+WALK_EEPROM_LENGTH;i++)
    if(ram_in_range(i,address,length))
      ram_data[i]=data[i-address];
}

